/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#include "FPSCounter.h"


bool FPSCounter::loadFPSShader(ShaderProgram* pShaderProgram)
{
    uiFPSProgram = glCreateProgram();
    pShaderProgram->loadShader(uiFPSProgram, VERTSHADER, strlen(VERTSHADER),
                               GL_VERTEX_SHADER);
    pShaderProgram->loadShader(uiFPSProgram, FRAGSHADER, strlen(FRAGSHADER),
                               GL_FRAGMENT_SHADER);
    pFont->setShaderProgram(uiFPSProgram);
    return true;
}

void FPSCounter::timeStamp()
{
    mLastTime = mCurrentTime;
#ifdef _WIN32
    _LARGE_INTEGER tmp;
    QueryPerformanceCounter(&tmp);
    mCurrentTime = (double)tmp.QuadPart;
#else
#ifdef _linux_
    gettimeofday(&mCurrentTime, NULL);
#else
    mCurrentTime = clock();
#endif
#endif
}

int FPSCounter::getFPS()
{
    double renderTime = getExecutionTime();
    double FPS = 1 / renderTime;
    return (int)ceil(FPS);
}

double FPSCounter::getExecutionTime()
{
    double renderTime = 0;
    if (mLastTime != 0)
    {
#ifdef _WIN32
        renderTime = (double)(mCurrentTime - mLastTime) / dqFreq;
#else
#ifdef _linux_
        renderTime = (mCurrentTime.tv_sec - mLastTime.tv_sec) * 1000000
                     + (mCurrentTime.tv_usec - mLastTime.tv_usec);
#else
        renderTime = (double)(mCurrentTime - mLastTime) / CLOCKS_PER_SEC;
#endif
#endif
    }
    return renderTime;
}

void FPSCounter::printFPS(glTFViewport* pViewport)
{
    glUseProgram(uiFPSProgram);
    glDisable(GL_DEPTH_TEST);
    GLuint iLoc = glGetUniformLocation(uiFPSProgram, "matrices.projMatrix");
    const glm::mat4 aMat = glm::ortho(0.0f, float(pViewport->width), 0.0f,
                                      float(pViewport->height));
    glUniformMatrix4fv(iLoc, 1, false, glm::value_ptr(aMat));
    glm::vec4 lightVector = glm::vec4(1.0f, 1.0f, 1.0f, 1.0f);
    iLoc = glGetUniformLocation(uiFPSProgram, "vColor");
    glUniform4fv(iLoc, 1, (GLfloat*)&lightVector);
    if (mDurationTime >= CLOCKS_PER_SEC)
    {
        mFPS = getFPS();
        mDurationTime = 0;
#ifdef _WIN32
        mTotalTime = getExecutionTime();
#endif

#ifdef _linux_
        mTotalTime = getExecutionTime();
#endif

#if WRITEFPS2FILE
        timeCounter++;
        if (timeCounter >= TIMETHRESHOLD)
        {
            FPSFile << "FPS:" << mFPS << "\n";
#ifdef _WIN32
            FPSFile << "Render Total Time: " << mTotalTime * 1000 << " ms\n";
#endif
            timeCounter = 0;
        }
#endif
    }
    else
    {
        if (mLastTime != 0)
        {
#ifdef _WIN32
            mDurationTime += ((mCurrentTime - mLastTime) / dqFreq) * 1000;
#else
#ifdef _linux_
            mDurationTime += (mCurrentTime - mLastTime) * 1000;
#else
            mDurationTime += mCurrentTime - mLastTime;
#endif
#endif
        }
    }

    pFont->printFormattedString(20, pViewport->height - 30, 20, "FPS: %d",
                                mFPS);
    pFont->printFormattedString(20, pViewport->height - 60, 20,
                                "Render Total Time: %.6f ms",
                                mTotalTime * 1000);
    glEnable(GL_DEPTH_TEST);
}
